/*
 * Copyright (c) 2006, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry.servlet;

import org.apache.commons.fileupload.FileItem;
import org.apache.commons.fileupload.FileItemFactory;
import org.apache.commons.fileupload.RequestContext;
import org.apache.commons.fileupload.disk.DiskFileItemFactory;
import org.apache.commons.fileupload.servlet.ServletFileUpload;
import org.apache.commons.fileupload.servlet.ServletRequestContext;
import org.wso2.registry.Resource;
import org.wso2.registry.RegistryConstants;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.Iterator;
import java.util.List;

public class FileUploadUtil {
    //TODO , we need to change the method signature
    public static Resource processUpload(HttpServletRequest req)
            throws IOException, ServletException {
        RequestContext reqContext = new ServletRequestContext(req);
        boolean isMultipart = ServletFileUpload.isMultipartContent(reqContext);
        if (isMultipart) {

            Resource resource = new Resource();

            try {
                //Create a factory for disk-based file items
                FileItemFactory factory = new DiskFileItemFactory();
                //Create a new file upload handler
                ServletFileUpload upload = new ServletFileUpload(factory);
                List items = upload.parseRequest(req);
                // Process the uploaded items
                Iterator iter = items.iterator();

                String parentPath = "";
                String resourceName = "";

                while (iter.hasNext()) {
                    FileItem item = (FileItem) iter.next();

                    if (item.isFormField()) {
                        if (item.getFieldName().equalsIgnoreCase("path")) {
                            String currentPath = (String)req.getSession().getAttribute("path");
                            parentPath = item.getString();

                        } else if (item.getFieldName().equalsIgnoreCase("filename")) {
                            resourceName = item.getString();

                        }else if (item.getFieldName().equalsIgnoreCase("description")) {
                            resource.setDescription(item.getString());
                        }

                    } else {
                        String fileName = item.getName();
                        String fileNameOnly;
                        if (fileName.indexOf("\\") < 0) {
                            fileNameOnly =
                                    fileName.substring(fileName.lastIndexOf("/") + 1, fileName
                                            .length());
                        } else {
                            fileNameOnly =
                                    fileName.substring(fileName.lastIndexOf("\\") + 1, fileName
                                            .length());
                        }
                        InputStream in = item.getInputStream();

                        byte[] buf = new byte[3000];
                        int read;
                        ByteArrayOutputStream out = new ByteArrayOutputStream();
                        while ((read = in.read(buf)) > 0) {
                            out.write(buf, 0, read);
                        }
                        out.flush();
                        if (fileNameOnly.endsWith(".xml") ||
                                fileNameOnly.endsWith(".wsdl") ||
                                fileNameOnly.endsWith("xsd")) {
                            resource.setMediaType("text/xml");
                        } else if (fileNameOnly.endsWith(".txt")) {
                            resource.setMediaType("text/html");
                        } else {
                            resource.setMediaType("undefined");
                        }
                        resource.setContent(out.toByteArray());
                    }

                    resource.setPath(calcualtePath(parentPath, resourceName));
                }
            } catch (Exception e) {
                req.setAttribute("status", "failure");
                req.setAttribute("cause", e.getMessage());

            }

            return resource;
        }
        return null;
    }

    private static String calcualtePath(String parentPath, String resourceName) {

        String resourcePath = null;

        if (!parentPath.startsWith(RegistryConstants.PATH_SEPARATOR)) {
            parentPath = RegistryConstants.PATH_SEPARATOR + parentPath;
        }

        if (parentPath.endsWith(RegistryConstants.PATH_SEPARATOR)) {
            resourcePath = parentPath + resourceName;
        } else {
            resourcePath = parentPath + RegistryConstants.PATH_SEPARATOR + resourceName;
        }

        return resourcePath;
    }

    /**
     * Get an absolute path from a current path and a target path.  If
     * the target path begins with a "/" it's absolute, so just use it.
     * Otherwise it's a relative path from currentPath.
     *
     * @param currentPath our current registry path
     * @param path the target path (may be relative or absolute)
     * @return an absolute path
     */
    private static String getAbsolutePath(String currentPath, String path) {
        if (path == null || path.length() == 0) return null;
        if (path.charAt(0) == '/') return path;
        if (currentPath == null || "/".equals(currentPath)) return "/" + path;
        return currentPath + "/" + path;
    }
}
