/*
 * Copyright (c) 2006, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry.web;

import org.wso2.registry.servlet.Utils;
import org.wso2.registry.servlet.FileUploadUtil;
import org.wso2.registry.Registry;
import org.wso2.registry.Resource;
import org.wso2.registry.RegistryConstants;
import org.wso2.registry.web.UIConstants;
import org.wso2.registry.web.actions.*;
import org.wso2.registry.web.ActionInvoker;

import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.ServletException;
import javax.servlet.ServletConfig;
import java.io.IOException;

public class ControllerServlet extends HttpServlet {

    protected transient ServletConfig servletConfig;

    //To store the conetxt path
    private String contextRoot = null;

    private ActionInvoker actionInvoker = new ActionInvoker();

    protected void doPost(HttpServletRequest request,
                          HttpServletResponse response)
            throws ServletException, IOException {
        initContextRoot(request);

        String uri = request.getRequestURI();
        String controlPart = uri.substring(contextRoot.length(), uri.length());

        if (controlPart.startsWith(RegistryConstants.PATH_SEPARATOR + UIConstants.SYSTEM_PATH)) {

            // process system requests

            String command = uri.substring((contextRoot + RegistryConstants.PATH_SEPARATOR + UIConstants.SYSTEM_PATH).
                    length(), uri.length());

            if (command.equals("/signin")) {

                SignInAction signInAction = new SignInAction();
                signInAction.setUserName(request.getParameter("userName"));
                signInAction.setPassword(request.getParameter("password"));

                try {
                    signInAction.execute(request);
                } catch (Exception e) {
                    e.printStackTrace();
                }

                String currentPath = (String) request.getSession().getAttribute(UIConstants.PATH_ATTR);
                if (currentPath == null) {
                    currentPath = RegistryConstants.ROOT_PATH;
                }

                forwardToResources(request, response, currentPath);

            } else if (command.equals("/signout")) {

                SignInAction signInAction = new SignInAction();
                signInAction.setUserName(RegistryConstants.ANONYMOUS_USER);
                signInAction.setPassword(request.getParameter(RegistryConstants.ANONYMOUS_PASSWORD));

                try {
                    signInAction.execute(request);
                } catch (Exception e) {
                    e.printStackTrace();
                }

                String currentPath = (String) request.getSession().getAttribute(UIConstants.PATH_ATTR);
                if (currentPath == null) {
                    currentPath = RegistryConstants.ROOT_PATH;
                }

                forwardToResources(request, response, currentPath);

            } else if (command.equals("/addUser")) {

                AddUserAction addUserAction = new AddUserAction();
                addUserAction.setNewUserName(request.getParameter("newUserName"));
                addUserAction.setPassword(request.getParameter("password"));
                addUserAction.setConfirmedPassword(request.getParameter("confirmedPassword"));
                addUserAction.setRoleName(request.getParameter("roleName"));

                try {
                    addUserAction.execute(request);
                } catch (Exception e) {
                    e.printStackTrace();
                }

                forwardToUserManagement(request, response);

            } else if (command.equals("/addRole")) {

                AddRoleAction addRoleAction = new AddRoleAction();
                addRoleAction.setNewRoleName(request.getParameter("newRoleName"));
                try {
                    addRoleAction.execute(request);
                } catch (Exception e) {
                    e.printStackTrace();
                }

                forwardToUserManagement(request, response);
            }
        }

        if (controlPart.equals("/web/addResource")) {
            handleFileUpload(request, response);
        }
    }

    private void handleFileUpload(HttpServletRequest request, HttpServletResponse response)
            throws IOException, ServletException {
        try {
            Registry registry = Utils.getSecureRegistry(request);

            Resource fileElement =
                    FileUploadUtil.processUpload(request);
            String path = fileElement.getPath();

            registry.put(path, fileElement);

            response.setContentType("text/html");

            String parentPath = getParentPath(path);
            ResourceDetailsAction details = new ResourceDetailsAction();
            details.setPath(parentPath);
            details.execute(request);

            CollectionViewAction collection = null;
            if (details.isCollection()) {
                collection = new CollectionViewAction();
                collection.setPath(parentPath);
                collection.execute(request);
            }

            request.getSession().setAttribute(UIConstants.RESOURCE_BEAN, details);
            request.getSession().setAttribute(UIConstants.COLLECTION_BEAN, collection);

            request.getRequestDispatcher(UIConstants.RESOURCES_JSP)
                    .forward(request, response);

        } catch (Exception e) {

            e.printStackTrace();
            response.setContentType("text/html");
            request.getRequestDispatcher(UIConstants.RESOURCES_JSP)
                    .forward(request, response);
        }
    }

    protected void doGet(HttpServletRequest request,
                         HttpServletResponse response)
            throws ServletException, IOException {

        initContextRoot(request);

        String uri = request.getRequestURI();

        String controlPart = uri.substring(contextRoot.length());
        if (controlPart.startsWith(RegistryConstants.PATH_SEPARATOR + UIConstants.WEB_PATH)) {

            // process request regarding resource handling

            String path;
            if (uri.equals("") || uri.endsWith(UIConstants.WEB_PATH + RegistryConstants.PATH_SEPARATOR)) {
                path = RegistryConstants.ROOT_PATH;

            } else {
                path = uri.substring((contextRoot + RegistryConstants.PATH_SEPARATOR + UIConstants.WEB_PATH).
                        length(), uri.length());
            }

            request.getSession().setAttribute(UIConstants.PATH_ATTR, path);

            actionInvoker.invokeActions(request);

            try {
                ResourceDetailsAction details = new ResourceDetailsAction();
                details.setPath(path);
                details.execute(request);

                CollectionViewAction collection = null;
                if (details.isCollection()) {
                    collection = new CollectionViewAction();
                    collection.setPath(path);
                    collection.execute(request);
                }

                request.getSession().setAttribute(UIConstants.RESOURCE_BEAN, details);
                request.getSession().setAttribute(UIConstants.COLLECTION_BEAN, collection);

                request.getRequestDispatcher(UIConstants.RESOURCES_JSP)
                        .forward(request, response);

            } catch (Exception e) {
                e.printStackTrace();
            }

        }  else if (controlPart.startsWith(RegistryConstants.PATH_SEPARATOR + UIConstants.SYSTEM_PATH)) {

            // process system requests

            String command = uri.substring((contextRoot + RegistryConstants.PATH_SEPARATOR + UIConstants.SYSTEM_PATH).
                    length(), uri.length());

            if (command.equals("/people")) {
                forwardToUserManagement(request, response);

            } else if (command.equals("/signout")) {

                SignInAction signInAction = new SignInAction();
                signInAction.setUserName(RegistryConstants.ANONYMOUS_USER);
                signInAction.setPassword(RegistryConstants.ANONYMOUS_PASSWORD);

                try {
                    signInAction.execute(request);
                } catch (Exception e) {
                    e.printStackTrace();
                }

                String currentPath = (String) request.getSession().getAttribute(UIConstants.PATH_ATTR);
                if (currentPath == null) {
                    currentPath = RegistryConstants.ROOT_PATH;
                }

                forwardToResources(request, response, currentPath);

            } else if (command.equals("/activity")) {

                forwardToActivity(request, response);

            } else if (command.equals("/activity/new")) {

                forwardToEmptyActivity(request, response);

            }
        }

    }

    /**
     * To find the name of the war , so getting that from the request context path
     *
     * @param req :  HttpServletRequest
     */
    public void initContextRoot(HttpServletRequest req) {
        if (contextRoot != null && contextRoot.trim().length() != 0) {
            return;
        }
        String contextPath = req.getContextPath();
//handling ROOT scenario, for servlets in the default (root) context, this method returns ""
        if (contextPath != null && contextPath.length() == 0) {
            contextPath = "/";
        }
        this.contextRoot = contextPath;
    }

    private String getParentPath(String resourcePath) {

        String parentPath;
        if (resourcePath.equals(RegistryConstants.ROOT_PATH)) {
            parentPath = null;
        } else {
            String formattedPath = resourcePath;
            if (resourcePath.endsWith(RegistryConstants.PATH_SEPARATOR)) {
                formattedPath = resourcePath.substring(
                        0, resourcePath.length() - RegistryConstants.PATH_SEPARATOR.length());
            }

            if (formattedPath.lastIndexOf(RegistryConstants.PATH_SEPARATOR) == 0) {
                parentPath = RegistryConstants.ROOT_PATH;
            } else {
                parentPath = formattedPath.substring(
                        0, formattedPath.lastIndexOf(RegistryConstants.PATH_SEPARATOR));
            }
        }

        return parentPath;
    }

    private void forwardToResources(HttpServletRequest request, HttpServletResponse response, String path) {

        try {
            ResourceDetailsAction details = new ResourceDetailsAction();
            details.setPath(path);
            details.execute(request);

            CollectionViewAction collection = null;
            if (details.isCollection()) {
                collection = new CollectionViewAction();
                collection.setPath(path);
                collection.execute(request);
            }

            request.getSession().setAttribute(UIConstants.RESOURCE_BEAN, details);
            request.getSession().setAttribute(UIConstants.COLLECTION_BEAN, collection);

            request.getRequestDispatcher(UIConstants.RESOURCES_JSP)
                    .forward(request, response);

        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private void forwardToUserManagement(HttpServletRequest request, HttpServletResponse response) {

        UserManagementAction userManagementAction = new UserManagementAction();
        try {
            userManagementAction.execute(request);
        } catch (Exception e) {
            e.printStackTrace();
        }

        request.getSession().setAttribute(UIConstants.USER_MANAGEMENT_BEAN, userManagementAction);

        try {
            request.getRequestDispatcher(UIConstants.USER_MANAGEMENT_JSP).forward(request, response);

        } catch (ServletException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    private void forwardToActivity(HttpServletRequest request, HttpServletResponse response) {

        RecentActivityAction recentActivityAction = new RecentActivityAction();
        recentActivityAction.setUserName(request.getParameter("userName"));
        recentActivityAction.setResourcePath(request.getParameter("path"));
        recentActivityAction.setFromDate(request.getParameter("fromDate"));
        recentActivityAction.setToDate(request.getParameter("toDate"));
        recentActivityAction.setFilter(request.getParameter("filter"));

        try {
            recentActivityAction.execute(request);
        } catch (Exception e) {
            e.printStackTrace();
        }

        request.getSession().setAttribute(UIConstants.ACTIVITY_BEAN, recentActivityAction);

        try {
            request.getRequestDispatcher(UIConstants.ACTIVITY_JSP).forward(request, response);

        } catch (ServletException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    private void forwardToEmptyActivity(HttpServletRequest request, HttpServletResponse response) {

        RecentActivityAction recentActivity = new RecentActivityAction();
        request.getSession().setAttribute(UIConstants.ACTIVITY_BEAN, recentActivity);

        try {
            request.getRequestDispatcher(UIConstants.ACTIVITY_JSP).forward(request, response);
        } catch (ServletException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }
}
